****************************************************************
*
*
*		128-TAP ECHO-CANCELLER PROGRAM
*		(C) COPYRIGHT TEXAS INSTRUMENTS INC., 1985
*
****************************************************************


	.title 'EC128'

*
*	ALGORITHM CONSTANTS
*

GAIN:	.equ	3		;* COEFF UPDDATE GAIN = 2**-10
*				*   = 2**(GAIN - 13)
LTAU:	.equ	9		;* LPF LONG TAU = 16 MSECS
*				*   = 125 USECS * 2**(16-LTAU)
STAU:	.equ	11		;* LPF SHORT TAU = 4 MSECS
*				*   = 125 USECS * 2**(16-STAU)
HTAU:	.equ	13		;* HPF TAU = 1MSEC = 1/2*PI*170HZ
*				*   = 125 USECS * 2**(16-HTAU)
THRES0: .equ	0800h		;* SUPRESS THRES = 1/16 (-24DB)
*				*   = 2**-THRES0 (-6DB * THRES0)
HANGT0: .equ	600		;* HANG OVER TIME = 75 MSECS
*				*   = 125 USECS * HANGT0
NER:	.equ	1		;* NEAR END SPEECH THRES = -6DB
*				*   = -6DB * NER
CUTOF0: .equ	021h		;* UPDATE CUTOFF = -48DB
*				*   = -6DB * (13 - LN(CUTOF0))

*
*	PAGE 0 DATA MEMORY ALLOCATION
*

P0DM:	.equ	0		;* PAGE 0 DATA MEM ADRS

DRR:	.equ	0		;* SERIAL PORT DATA RECEIVE REG
DXR:	.equ	1		;* SERIAL PORT DATA TRANSMIT REG
TIM:	.equ	2		;* TIMER REG (NOT USED)
PRD:	.equ	3		;* PERIOD REG (NOT USED)
IMR:	.equ	4		;* INTERRUPT MASK REG
GREG:	.equ	5		;* MEM ALLOCATION REG (NOT USED)

DRR1:	.equ	96		;* STORAGE FOR S(0) BY RINT
DRR2:	.equ	97		;* STORAGE FOR Y(0) BY RINT
TST0:	.equ	98		;* STORAGE FOR ST0 BY RINT
TACCH:	.equ	99		;* STORAGE FOR ACC HIGH BY RINT
TACCL:	.equ	100		;* STORAGE FOR ACC LOW BY RINT

BADDR:	.equ	101		;* BASE ADR FOR MU-LAW EXPANSION
BIAS2:	.equ	102		;* BIAS FOR MU-LAW COMPRESSION
NEG7:	.equ	103		;* -7 FOR MU-LAW COMPRESSION
Q:	.equ	104		;* MU-LAW COMPRESSION MANTISSA
SI:	.equ	105		;* MU-LAW COMPRESSION SIGN

TEMP1:	.equ	104		;* TEMPORARY STORAGE LOCATION 1
TEMP2:	.equ	105		;* TEMPORARY STORAGE LOCATION 2

EEST:	.equ	107		;* ECHO ESTIMATE
OUTPUT: .equ	108		;* OUTPUT ESTIMATE
THRES:	.equ	109		;* RESIDUAL OUTPUT SPRS THRESHOLD

ONE:	.equ	110		;* HOLDS 1

ADY142: .equ	111		;* Y142 DATA MEM ADRS

S0:	.equ	112		;* NEAR-END SAMPLE MSBS
S0LSBS: .equ	113		;* NEAR-END SAMPLE LSBS
S0DC:	.equ	114		;* INPUT NEAR-END SAMPLE (K=0)
S1DC:	.equ	115		;* INPUT NEAR-END SAMPLE (K=1)

*
*	PAGE 4 DATA MEMORY ALLOCATION
*

P4DM:	.equ	512		;* PAGE 4 DATA MEM ADRS
P4PM:	.equ	65280		;* PAGE 4 PROG MEM ADRS

UN0:	.equ	0		;* NORMALIZED OUTPUT (K=0)
UN15:	.equ	15		;* NORMALIZED OUTPUT (K=15)

*
*	PAGE 5 DATA MEMORY ALLOCATION
*

P5DM:	.equ	640		;* PAGE 5 DATA MEM ADRS
P5PM:	.equ	65408		;* PAGE 5 PROG MEM ADRS

A127:	.equ	0		;* FIR FILTER COEFFICIENT (K=127)
A0:	.equ	127		;* FIR FILTER COEFFICIENT (K=0)

*
*	PAGE 6 DATA MEMORY ALLOCATION
*

P6DM:	.equ	768		;* PAGE 6 DATA MEM ADRS

Y0:	.equ	0		;* REFERENCE SAMPLE (K=0)
Y127:	.equ	127		;* REFERENCE SAMPLE (K=127)

*
*	PAGE 7 DATA MEMORY ALLOCATION
*

P7DM:	.equ	896		;* PAGE 7 DATA MEM ADRS

Y128:	.equ	0		;* REFERENCE SAMPLE (K=128)
Y143:	.equ	15		;* REFERENCE SAMPLE (K=143)

TEMP3:	.equ	16		;* TEMPORARY STORAGE LOCATION 3

CUN0:	.equ	17		;* COPY OF UN0 FROM PAGE 4

AONE:	.equ	18		;* HOLDS 1
SONE:	.equ	19		;* HOLDS SATURATION 1

ADA0:	.equ	20		;* A0 DATA MEM ADRS
ADY1:	.equ	21		;* Y1 DATA MEM ADRS
ADINC0: .equ	22		;* INC0 DATA MEM ADRS
ADM7:	.equ	23		;* M7 DATA MEM ADRS
ADUN14: .equ	24		;* UN14 DATA MEM ADRS

H:	.equ	96		;* MODULO 16 COUNTER

HANGT:	.equ	97		;* HANG OVER COUNTER RESET VALUE
HCNTR:	.equ	98		;* HANG OVER COUNTER

ABSS0:	.equ	99		;* |S0|
ABSS0F: .equ	100		;* SHORT TAU LPF 2*|S0|

ABSE0:	.equ	101		;* |OUTPUT|
ABSOUT: .equ	102		;* LONG TAU LPF |OUTPUT| MSBS
AELSBS: .equ	103		;* LONG TAU LPF |OUTPUT| LSBS

ABSY0:	.equ	104		;* |Y0|
ABSY:	.equ	105		;* LONG TAU LPF |Y0| MSBS
AYLSBS: .equ	106		;* LONG TAU LPF |Y0| LSBS
IABSY:	.equ	107		;* 1/ABSY
CUTOFF: .equ	108		;* ABSY CUTOFF LVL FOR NO UPDATE
ABSY0F: .equ	109		;* SHORT TAU LPF |Y0|

M0:	.equ	110		;* LOCAL MAXIMA (K=0)
M8:	.equ	118		;* LOCAL MAXIMA (K=8)

INC0:	.equ	120		;* UPDATE INCREMENT (K=0)
INC7:	.equ	127		;* UPDATE INCREMENT (K=7)


A127PM: .equ	P5PM+A127	;* A127 PROG MEM ADRS
UN0PM:	.equ	P4PM+UN0	;* UN0 PROG MEM ADRS


****************************************************************
*
*		INTERRUPT BRANCHES
*
****************************************************************


	.sect	"AORG0"

	B	INIT		;* ON HARDWARE RESET GO TO INIT


	.sect	"AORG1"

	B	RXRT		;* ON RINT GO TO RXRT

	B	TXRT		;* ON TINT GO TO TXRT
	.page
****************************************************************
*
*		PROCESSOR INITIALIZATION ROUTINE
*
****************************************************************


	.sect	"AORG2"

INIT	LDPK	0		;* INTIALIZE ST0 AND ST1

	LALK	02e00h		;* 0010 1110 0000 0000	IN BINARY
	SACL	TEMP1		;* DATA FOR ST0

	LST	TEMP1		;* 0 -> DP    PG POINTER SET TO 0
*				* 1 -> INTM  INTERRUPTS DISABLED
*				* 1 -> OVM   OVERFLOW SATURATION
*				* 0 -> OV    OVERLOW REG CLEARED
*				* 1 -> ARP   AR POINTER SET TO 1

	LALK	027f8h		;* 0010 0111 1111 1000	IN BINARY
	SACL	TEMP1		;* DATA FOR ST1

	LST1	TEMP1		;* 0 -> PM    NO P REG SHIFTING
*				* 0 -> TXM   FSX IS AN INPUT
*				* 1 -> F0    DRR, DXR TO 8 BITS
*				* 1 -> XF    XF PIN SET TO HIGH
*				* 1 -> SXM   SIGN EXTENSION ON
*				* 0 -> TC    TC FLAG BIT RESET
*				* 0 -> CNF   B0 IS DATA MEM
*				* 1 -> ARB   AND  1 -> ARP

*
*	INITIALIZE PAGE 0
*

	LARK	AR1,96		;* LOWEST PAGE 0 LOCATION -> AR1

	ZAC	;* 0 -> ACC

	RPTK	31

	SACL	*+		;* ZERO PAGE 0

	LALK	030h		;* ENABLE  XINT,RINT
	SACL	IMR		;* DISBALE TINT,INT0,INT1,INT2

	LALK	0ffffh		;* MU-LAW FFFF = LINEAR 0
	SACL	DXR

	SACL	DRR1

	SACL	DRR2

	LALK	XTBL
	SACL	BADDR

	LALK	132
	SACL	BIAS2

	LALK	-7
	SACL	NEG7

	LALK	THRES0
	SACL	THRES

	LALK	1
	SACL	ONE

	LALK	P7DM+Y143-1
	SACL	ADY142


*
*	INITIALIZE PAGE 4 AND 5
*

	LRLK	AR1,512 	;* LOWEST PAGE 4 ADDRESS -> AR1

	ZAC	;* 0 -> ACC

	RPTK	255

	SACL	*+		;* ZERO PAGE 4 AND 5

*
*	INITIALIZE PAGE 6 AND 7
*

	RPTK	255

	SACL	*+		;* ZERO PAGE 6 AND 7

	LDPK	7

	LALK	1
	SACL	AONE

	LALK	04fffh
	SACL	SONE

	LALK	P5DM+A0
	SACL	ADA0

	LALK	P6DM+Y0+1
	SACL	ADY1

	LALK	P7DM+INC0
	SACL	ADINC0

	LALK	P7DM+M8-1
	SACL	ADM7

	LALK	P4DM+UN15-1
	SACL	ADUN14

	LALK	HANGT0
	SACL	HANGT

	LALK	0400h		;* >400 = 1/8 OF MAX ABSY
	SACL	ABSY

	LALK	020h
	SACL	IABSY

	LALK	CUTOF0
	SACL	CUTOFF

	LALK	0400h
	SACL	ABSY0F

	SACL	M0

	EINT

	B LOOP
	.page
****************************************************************
*
*		CYCLE START ROUTINE
*
****************************************************************

START	LDPK	0

*
*	CONVERT MU-LAW INPUT REFERENCE SAMPLE TO LINEAR  (Y0)
*

	ZALS	DRR2		;* MU-LAW Y(0) -> ACC
	ADD	BADDR		;* ADD MU-LAW TABLE BASE ADDRESS
	LDPK	6
	TBLR	Y0		;* LINEAR Y(0) -> Y0

*
*	COMPUTE ABSOLUTE VALUE OF Y0
*

	LAC	Y0		;* Y0 -> ACC
	ABS
	LDPK	7
	SACL	ABSY0		;* |Y0| -> ABSY0 ON PAGE 7
	LDPK	0

*
*	CONVERT MU-LAW NEAR END SAMPLE TO LINEAR (S0DC)
*

	ZALS	DRR1		;* MU-LAW S(0)DC -> ACC
	ADD	BADDR		;* ADD MU-LAW TABLE BASE ADDRESS
	TBLR	S0DC		;* LINEAR S(0)DC -> S0DC
*
*	COMPUTE HIGH PASS FILTERED NEAR END SAMPLE (S0)
*
	ZALS	S0LSBS		;* S0LSBS -> LOW ACC
	ADDH	S0		;* S0 (MSBS) -> HIGH ACC
	SUB	S0,HTAU 	;* ACC - S0 * 2**HTAU -> ACC
	ADDH	S0DC		;* ACC + S0DC * 2**16 -> ACC
	SUB	S0DC,HTAU-1	;* ACC - S0DC * 2**HTAU-1 -> ACC
	SUBH	S1DC		;* ACC - S1DC * 2**16 -> ACC
	ADD	S1DC,HTAU-1	;* ACC + S1DC * 2**HTAU-1 -> ACC
	SACL	S0LSBS		;* LOW ACC -> S0LSBS
	SACH	S0		;* HIGH ACC -> S0 (MSBS)
	DMOV	S0DC		;* S0DC -> S1DC
*
*	COMPUTE ABSOLUTE VALUE OF S0
*
	ABS
	LDPK	7
	SACH	ABSS0		;* |S0| -> ABSS0 ON PAGE 7
	.page
****************************************************************
*
*		ECHO ESTIMATION ROUTINE
*
****************************************************************

EESTR	LDPK	0
*
*	MOVE Y128,Y129,...,Y142 TO NEXT HIGHER MEMORY LOCATION
*
	LARP	AR1		;* 1 -> AR POINTER
	LAR	AR1,ADY142	;* ADY142 -> AR1
	RPTK	14		;* K = 142,141,...,128
	DMOV	*-		;* Y(K) -> Y(K+1)
*
*	CONVOLVE REFERENCE SAMPLES WITH FIR COEFFICIENTS
*
FIR	LAC	ONE,14		;* ROUND-OFF OFFSET -> ACC
	MPYK	0		;* 0 -> P
	CNFP	;* AR1 STILL POINTS AT Y127
	RPTK	127		;* K = 127,126,...,0
	MACD	A127PM,*-	;* Y(K) * A(I-K) + ACC -> ACC
	CNFD
	APAC	;* P + ACC -> ACC
	SACH	EEST,1		;* 2 * HIGH ACC -> EEST
*
*	COMPUTE THE OUTPUT
*
	LAC	S0		;* S0 -> ACC
	SUB	EEST		;* ACC - EEST -> ACC
	SACL	OUTPUT		;* ACC -> OUTPUT
	LDPK	7
	ABS
	SACL	ABSE0		;* ACC -> ABSE0 ON PAGE 7
	.page
****************************************************************
*
*		RESIDUAL OUTPUT SUPPRESSION ROUTINE
*
****************************************************************


SPRS	LT	IABSY		;* IABSY -> T REG
	MPY	ABSOUT		;* ABSOUT * IABSY -> P REG
	LAC	HCNTR		;* NEAR END SPEECH FLAG -> ACC
	LDPK	0
	BGZ	WOUT		;* IF N.E. SPEECH NO SPRS
	PAC	;* P REG -> ACC
	SUB	THRES		;* ACC - THRES -> ACC
	BGZ	WOUT		;* IF THRES EXCEEDED SKIP SPRS
	BIOZ	WOUT		;* IF BIO PIN LOW SKIP SPRS
	ZAC	;* 0 -> ACC
	SACL	OUTPUT		;* ACC -> OUTPUT
WOUT	LT	OUTPUT		;* OUTPUT -> T REG   (FOR UN0)
	.page
****************************************************************
*
*		LINEAR TO MU-LAW COMPRESSION ROUTINE
*
****************************************************************

CMPRS	ZALH	OUTPUT		;* OUTPUT -> ACC
	SFL
	SFL	;* LEFT JUSTIFY ACC
	BLZ	NEGCMP		;* IF ACC < 0 THEN GO TO NEGCMP

POSCMP	ADDH	BIAS2
	LAR	AR1,NEG7
	RPTK	6		;* FIND MSB
	NORM
	ANDK	0f000h,14	;* ZERO 2 MSBS AND ALL LSBS
	SACH	Q
	SAR	AR1,SI
	ZALH	SI
	ABS
	ADD	Q,2
	XORK	0ff00h,4	;* INVERT ALL BITS
	B	TXOUT

NEGCMP	ABS	;* LEFT JUSTIFIED OUTPUT IN ACC
	ADDH	BIAS2
	LAR	AR1,NEG7
	RPTK	6		;* FIND MSB
	NORM
	ANDK	0f000h,14	;* ZERO 2 MSBS AND ALL LSBS
	SACH	Q
	SAR	AR1,SI
	ZALH	SI
	ABS
	ADD	Q,2
	XORK	07f00h,4	;* INVERT ALL BITS IN Q

TXOUT	SACH	DXR,4		;* 2**4 * HIGH ACC -> DXR
	.page
****************************************************************
*
*		POWER ESTIMATION ROUTINE
*
****************************************************************

NORM	LDPK	7		;* T REG STILL CONTAINS OUTPUT
*
*	UPDATE LONG TAU OUTPUT POWER ESTIMATE (ABSOUT)
*
	ZALH	ABSOUT		;* ABSOUT -> HIGH ACC
	ADDS	AELSBS		;* AELSBS -> LOW ACC
	SUB	ABSOUT,LTAU	;* ACC - ABSOUT * 2**LTAU -> ACC
	ADD	ABSE0,LTAU	;* ACC + ABSE0 * 2**LTAU -> ACC
	SACH	ABSOUT		;* HIGH ACC -> ABSOUT
	SACL	AELSBS		;* LOW ACC -> AELSBS
*
*	UPDATE LONG TAU REFERENCE POWER ESTIMATE (ABSY)
*
	ZALH	ABSY		;* ABSY -> HIGH ACC
	ADDS	AYLSBS		;* AYLSBS -> LOW ACC
	SUB	ABSY,LTAU	;* ACC - ABSY * 2**LTAU -> ACC
	ADD	ABSY0,LTAU	;* ACC + ABSY0 * 2**LTAU -> ACC
	ADD	CUTOFF,LTAU	;* ACC + CUTOFF * 2**LTAU -> ACC
	SACH	ABSY		;* HIGH ACC -> ABSY
	SACL	AYLSBS		;* LOW ACC -> AYLSBS
*
*	COMPUTE 1/ABSY (DIVIDE 1 BY ABSY)
*
	ZALH	AONE
	RPTK	14
	SUBC	ABSY
	SACL	IABSY
	.page
***************************************************************
*
*		OUTPUT NORMALIZATION ROUTINE
*
***************************************************************

*
*	MOVE UN0,UN1,...,UN14 TO NEXT HIGHER MEMORY LOCATION
*
	LAR	AR1,ADUN14	;* ADUN14 -> AR1
	RPTK	13		;* K=14,13,...,1
	DMOV	*-		;* UN(K) -> UN(K+1)
	DMOV	*		;* UN(0) -> UN(1)
*
*	COMPUTE NORMALIZED OUTPUT (UN0)
*
	MPY	IABSY		;* IABSY * T REG(OUTPUT) -> P REG
	PAC	;* P REG  (UN0) -> ACC
*
*	SATURATE NORMALIZED OUTPUT (UN0) AT +/- 1.0
*
	BGEZ	POSUN0		;* IF UN0 > 0 THEN GO TO POSUN0

NEGUN0	ADD	SONE		;* ACC + SONE -> ACC
	BGEZ	SMLUN0		;* IF -1.0 < UN0 < 0 THEN NO SATR

	ZAC	;* 0 -> ACC
	SUB	SONE		;* ACC - SONE -> ACC
	B	SAVUN0

POSUN0	SUB	SONE		;* ACC - SONE -> ACC
	BLEZ	SMLUN0		;* IF 0 < UN0 < 1.0 THEN NO SATR

	LAC	SONE		;* SONE -> ACC
	B	SAVUN0

SMLUN0	PAC	;* P REG  (UN0) -> ACC

SAVUN0	SACL	CUN0		;* ACC -> CUN0
	SACL	*		;* ACC -> UN(0)
	.page
***************************************************************
*
*		NEAR-END SPEECH DETECTION ROUTINE
*
***************************************************************

NESP	LDPK	7
*
*	UPDATE SHORT TAU REFERENCE POWER ESTIMATE (ABSY0F)
*
	ZALH	ABSY0F		;* ABSY0F * 2**16 -> ACC
	SUB	ABSY0F,STAU	;* ACC - ABSY0F * 2**STAU -> ACC
	ADD	ABSY0,STAU	;* ACC + ABSY0 * 2**STAU -> ACC
	SACH	ABSY0F		;* HIGH ACC -> ABSY0F
*
*	UPDATE SHORT TAU NEAR END POWER ESTIMATE (ABSS0F)
*
	ZALH	ABSS0F		;* ABSS0F * 2**16 -> ACC
	SUB	ABSS0F,STAU	;* ACC - ABSS0F * 2**STAU -> ACC
	ADD	ABSS0,STAU+NER	;* ACC + ABSS0*2**STAU+NER -> ACC
	SACH	ABSS0F		;* HIGH ACC -> ABSS0F
*
*	UPDATE MODULO 16 COUNTER (H)
*
	LAC	H		;* H -> ACC
	ADD	AONE		;* ACC + 1 -> ACC
	ANDK	0fh		;* IF ACC = 16 THEN 0 -> ACC
	SACL	H		;* ACC -> H
	BGZ	NESP1		;* IF H > 0 THEN GO TO NESP1
*
*	MOVE M0,M1,...,M7 TO NEXT HIGHER MEMORY LOCATION
*
	LAR	AR1,ADM7	;* ADM7 -> AR1
	RPTK	7		;* K=7,6,...,0
	DMOV	*-		;* M(K) -> M(K+1)
	DMOV	ABSY0F		;* ABSY0F -> M0
	B	NESP3		;* ON MEMORY MOVES SKIP DETECTION
*
*	UPDATE MOST RECENT LOCAL MAXIMA (M0)
*
NESP1	LAC	ABSY0F		;* ABSY0F -> ACC
	SUB	M0		;* ACC - M0 -> ACC
	BLEZ	NESP2		;* IF M0 > ABSY0F THEN NO UPDATE
	DMOV	ABSY0F		;* ABSY0F -> M0
*
*	COMPARE REFERENCE POWER TO NEAR-END POWER
*
NESP2	LAC	ABSS0F		;* ABSS0F -> ACC
	SUB	M0		;* ACC - M0 -> ACC
	BLEZ	NESP3		;* NO N.E. SPEECH IF M0 > ABSS0F

	LAC	ABSS0F		;* ABSS0F -> ACC
	SUB	M0+1		;* ACC - M1 -> ACC
	BLEZ	NESP3		;* NO N.E. SPEECH IF M1 > ABSS0F

	LAC	ABSS0F		;* ABSS0F -> ACC
	SUB	M0+2		;* ACC - M2 -> ACC
	BLEZ	NESP3		;* NO N.E. SPEECH IF M2 > ABSS0F

	LAC	ABSS0F		;* ABSS0F -> ACC
	SUB	M0+3		;* ACC - M3 -> ACC
	BLEZ	NESP3		;* NO N.E. SPEECH IF M3 > ABSS0F

	LAC	ABSS0F		;* ABSS0F -> ACC
	SUB	M0+4		;* ACC - M4 -> ACC
	BLEZ	NESP3		;* NO N.E. SPEECH IF M4 > ABSS0F

	LAC	ABSS0F		;* ABSS0F -> ACC
	SUB	M0+5		;* ACC - M5 -> ACC
	BLEZ	NESP3		;* NO N.E. SPEECH IF M5 > ABSS0F

	LAC	ABSS0F		;* ABSS0F -> ACC
	SUB	M0+6		;* ACC - M6 -> ACC
	BLEZ	NESP3		;* NO N.E. SPEECH IF M6 > ABSS0F

	LAC	ABSS0F		;* ABSS0F -> ACC
	SUB	M0+7		;* ACC - M7 -> ACC
	BLEZ	NESP3		;* NO N.E. SPEECH IF M7 > ABSS0F

	LAC	ABSS0F		;* ABSS0F -> ACC
	SUB	M0+8		;* ACC - M8 -> ACC
	BLEZ	NESP3		;* NO N.E. SPEECH IF M8 > ABSS0F
*
*	NEAR-END SPEECH DETECTED  SET HANGOVER COUNTER (HCNTR)
*
	DMOV	HANGT		;* HANGT -> HCNTR
*
*	CHECK AND UPDATE HANGOVER COUNTER
*
NESP3	LAC	HCNTR		;* HCNTR -> ACC
	BZ	NESP4		;* IF HCNTR = 0 THEN GO TO NESP4
	SUB	AONE		;* ACC - 1 -> ACC
	SACL	HCNTR		;* ACC -> HCNTR
	B	LOOP		;* GO TO CYCLE END
*
*	CHECK IF LTAU REFERENCE POWER ESTIMATE IS BELOW CUTOFF
*
NESP4	LAC	ABSY		;* ABSY -> ACC
	SUB	CUTOFF		;* ACC - CUTOFF -> ACC
	BLEZ	LOOP		;* IF ABSY < CUTOFF THEN LOOP
	.page
****************************************************************
*
*		COEFFICIENT INCREMENT UPDATE ROUTINE
*
****************************************************************

UPINC	LAC	ADY1		;* ADY1 -> ACC	(Y0 IS NOW IN Y1)
	ADD	H		;* ACC + H -> ACC
	SACL	TEMP3
	LAR	AR1,TEMP3	;* ADY1 + H -> AR1
	CNFP
	LT	CUN0		;* UN0 -> T REG

	LAC	AONE,15 	;* ROUND-OFF OFFSET -> ACC
	MPY	*+		;* UN(0) * Y(0+H) -> P REG
	RPTK	14		;* K = 1,2,...,15
	MAC	UN0PM+1,*+	;* UN(K) * Y(K+H) + ACC -> ACC
	LTA	CUN0		;* P REG + ACC -> ACC	UN0 -> T
	SACH	INC0		;* HIGH ACC -> INC(0)

	LAC	AONE,15 	;* ROUND-OFF OFFSET -> ACC
	MPY	*+		;* UN(0) * Y(16+H) -> P REG
	RPTK	14		;* K = 1,2,...,15
	MAC	UN0PM+1,*+	;* UN(K) * Y(K+16+H) + ACC -> ACC
	LTA	CUN0		;* P REG + ACC -> ACC	UN0 -> T
	SACH	INC0+1		;* HIGH ACC -> INC(1)

	LAC	AONE,15 	;* ROUND-OFF OFFSET -> ACC
	MPY	*+		;* UN(0) * Y(32+H) -> P REG
	RPTK	14		;* K = 1,2,...,15
	MAC	UN0PM+1,*+	;* UN(K) * Y(K+32+H) + ACC -> ACC
	LTA	CUN0		;* P REG + ACC -> ACC	UN0 -> T
	SACH	INC0+2		;* HIGH ACC -> INC(2)

	LAC	AONE,15
	MPY	*+
	RPTK	14
	MAC	UN0PM+1,*+
	LTA	CUN0
	SACH	INC0+3

	LAC	AONE,15
	MPY	*+
	RPTK	14
	MAC	UN0PM+1,*+
	LTA	CUN0
	SACH	INC0+4

	LAC	AONE,15
	MPY	*+
	RPTK	14
	MAC	UN0PM+1,*+
	LTA	CUN0
	SACH	INC0+5

	LAC	AONE,15
	MPY	*+
	RPTK	14
	MAC	UN0PM+1,*+
	LTA	CUN0
	SACH	INC0+6

	LAC	AONE,15
	MPY	*+
	RPTK	14
	MAC	UN0PM+1,*+
	LTA	CUN0
	SACH	INC0+7

	CNFD
	.page
****************************************************************
*
*	COEFFICIENT UPDATE ROUTINE
*
****************************************************************

	LARK	AR0,16		;* 16 -> AR0   (AR2 INCREMENT)
	LAR	AR1,ADINC0	;* ADINC0 -> AR1
	LAC	ADA0		;* ADA0 -> ACC
	SUB	H		;* ACC - H -> ACC
	SACL	TEMP3
	LAR	AR2,TEMP3	;* ADA0 - H -> AR2
	SPM	2		;* SET 4 BIT LEFT SHIFT OF P REG
	LAC	IABSY,GAIN	;* IABSY * 2**GAIN -> ACC
	SACL	TEMP3		;* ACC -> TEMP3
	LT	TEMP3		;* TEMP3 -> T REG

	MPY	*+,AR2		;* INC(0) * T REG -> P REG
	ZALH	*		;* A(H) * 2**16 -> ACC
	APAC	;* P REG + ACC -> ACC
	SACH	*0-,0,AR1	;* HIGH ACC -> A(H)

	MPY	*+,AR2		;* INC(1) * T REG -> P REG
	ZALH	*		;* A(16+H) * 2**16 -> ACC
	APAC	;* P REG + ACC -> ACC
	SACH	*0-,0,AR1	;* HIGH ACC -> A(16+H)

	MPY	*+,AR2		;* INC(2) * T REG -> P REG
	ZALH	*		;* A(32+H) * 2**16 -> ACC
	APAC	;* P REG + ACC -> ACC
	SACH	*0-,0,AR1	;* HIGH ACC -> A(32+H)

	MPY	*+,AR2
	ZALH	*
	APAC
	SACH	*0-,0,AR1

	MPY	*+,AR2
	ZALH	*
	APAC
	SACH	*0-,0,AR1

	MPY	*+,AR2
	ZALH	*
	APAC
	SACH	*0-,0,AR1

	MPY	*+,AR2
	ZALH	*
	APAC
	SACH	*0-,0,AR1

	MPY	*+,AR2
	ZALH	*
	APAC
	SACH	*0-,0,AR1

	SPM	0		;* SET NO SHIFT OF P REG
	.page
****************************************************************
*
*		CYCLE END ROUTINE
*
****************************************************************

LOOP	IDLE	;* WAIT IN LOOP UNITL RINT/XINT
	IDLE	;* EXTRA IDLE FOR TWO RINT
	NOP

****************************************************************
*
*		RINT SERVICE ROUTINE
*
****************************************************************

RXRT	SST	TST0		;* SAVE ST0
	LDPK	0		;* 0 -> PAGE POINTER
	SACH	TACCH		;* SAVE HIGH ACC
	SACL	TACCL		;* SAVE LOW ACC
	DMOV	DRR1		;* DRR1 -> DRR2
	ZALS	DRR		;* DRR -> ACC
	ANDK	0ffh		;* MASK-OFF MSB BYTE
	SACL	DRR1		;* ACC -> DRR1
	ZALS	TACCL		;* RESTORE LOW ACC
	ADDH	TACCH		;* RESTORE HIGH ACC
	LST	TST0		;* RESTORE ST0
	EINT	;* INTERRUPTS ENABLED
	RET	;* RETURN TO PROGRAM

****************************************************************
*
*		XINT SERVICE ROUTINE
*
****************************************************************

TXRT	EINT	;* INTERRUPTS ENABLED
	B	START		;* BRANCH TO PROGRAM START
	.page
****************************************************************
*
*	       MU-LAW EXPANSION LOOKUP TABLE
*
****************************************************************


	.sect	"AORG3"

XTBL:	.equ	 $

	.word  0e0a1h		;* NEGATIVE VALUES FIRST
	.word  0e1a1h		;*  (FF, FE, ETC.)
	.word  0e2a1h
	.word  0e3a1h
	.word  0e4a1h
	.word  0e5a1h
	.word  0e6a1h
	.word  0e7a1h
	.word  0e8a1h
	.word  0e9a1h
	.word  0eaa1h
	.word  0eba1h
	.word  0eca1h
	.word  0eda1h
	.word  0eea1h
	.word  0efa1h
	.word  0f061h
	.word  0f0e1h
	.word  0f161h
	.word  0f1e1h
	.word  0f261h
	.word  0f2e1h
	.word  0f361h
	.word  0f3e1h
	.word  0f461h
	.word  0f4e1h
	.word  0f561h
	.word  0f5e1h
	.word  0f661h
	.word  0f6e1h
	.word  0f761h
	.word  0f7e1h
	.word  0f841h
	.word  0f881h
	.word  0f8c1h
	.word  0f901h
	.word  0f941h
	.word  0f981h
	.word  0f9c1h
	.word  0fa01h
	.word  0fa41h
	.word  0fa81h
	.word  0fac1h
	.word  0fb01h
	.word  0fb41h
	.word  0fb81h
	.word  0fbc1h
	.word  0fc01h
	.word  0fc31h
	.word  0fc51h
	.word  0fc71h
	.word  0fc91h
	.word  0fcb1h
	.word  0fcd1h
	.word  0fcf1h
	.word  0fd11h
	.word  0fd31h
	.word  0fd51h
	.word  0fd71h
	.word  0fd91h
	.word  0fdb1h
	.word  0fdd1h
	.word  0fdf1h
	.word  0fe11h
	.word  0fe29h
	.word  0fe39h
	.word  0fe49h
	.word  0fe59h
	.word  0fe69h
	.word  0fe79h
	.word  0fe89h
	.word  0fe99h
	.word  0fea9h
	.word  0feb9h
	.word  0fec9h
	.word  0fed9h
	.word  0fee9h
	.word  0fef9h
	.word  0ff09h
	.word  0ff19h
	.word  0ff25h
	.word  0ff2dh
	.word  0ff35h
	.word  0ff3dh
	.word  0ff45h
	.word  0ff4dh
	.word  0ff55h
	.word  0ff5dh
	.word  0ff65h
	.word  0ff6dh
	.word  0ff75h
	.word  0ff7dh
	.word  0ff85h
	.word  0ff8dh
	.word  0ff95h
	.word  0ff9dh
	.word  0ffa3h
	.word  0ffa7h
	.word  0ffabh
	.word  0ffafh
	.word  0ffb3h
	.word  0ffb7h
	.word  0ffbbh
	.word  0ffbfh
	.word  0ffc3h
	.word  0ffc7h
	.word  0ffcbh
	.word  0ffcfh
	.word  0ffd3h
	.word  0ffd7h
	.word  0ffdbh
	.word  0ffdfh
	.word  0ffe2h
	.word  0ffe4h
	.word  0ffe6h
	.word  0ffe8h
	.word  0ffeah
	.word  0ffech
	.word  0ffeeh
	.word  0fff0h
	.word  0fff2h
	.word  0fff4h
	.word  0fff6h
	.word  0fff8h
	.word  0fffah
	.word  0fffch
	.word  0fffeh
	.word  00h

	.word  01f5fh		;* POSITIVE VALUES NEXT
	.word  01e5fh		;*  (POLARITY BIT = 1)
	.word  01d5fh
	.word  01c5fh
	.word  01b5fh
	.word  01a5fh
	.word  0195fh
	.word  0185fh
	.word  0175fh
	.word  0165fh
	.word  0155fh
	.word  0145fh
	.word  0135fh
	.word  0125fh
	.word  0115fh
	.word  0105fh
	.word  0f9fh
	.word  0f1fh
	.word  0e9fh
	.word  0e1fh
	.word  0d9fh
	.word  0d1fh
	.word  0c9fh
	.word  0c1fh
	.word  0b9fh
	.word  0b1fh
	.word  0a9fh
	.word  0a1fh
	.word  099fh
	.word  091fh
	.word  089fh
	.word  081fh
	.word  07bfh
	.word  077fh
	.word  073fh
	.word  06ffh
	.word  06bfh
	.word  067fh
	.word  063fh
	.word  05ffh
	.word  05bfh
	.word  057fh
	.word  053fh
	.word  04ffh
	.word  04bfh
	.word  047fh
	.word  043fh
	.word  03ffh
	.word  03cfh
	.word  03afh
	.word  038fh
	.word  036fh
	.word  034fh
	.word  032fh
	.word  030fh
	.word  02efh
	.word  02cfh
	.word  02afh
	.word  028fh
	.word  026fh
	.word  024fh
	.word  022fh
	.word  020fh
	.word  01efh
	.word  01d7h
	.word  01c7h
	.word  01b7h
	.word  01a7h
	.word  0197h
	.word  0187h
	.word  0177h
	.word  0167h
	.word  0157h
	.word  0147h
	.word  0137h
	.word  0127h
	.word  0117h
	.word  0107h
	.word  0f7h
	.word  0e7h
	.word  0dbh
	.word  0d3h
	.word  0cbh
	.word  0c3h
	.word  0bbh
	.word  0b3h
	.word  0abh
	.word  0a3h
	.word  09bh
	.word  093h
	.word  08bh
	.word  083h
	.word  07bh
	.word  073h
	.word  06bh
	.word  063h
	.word  05dh
	.word  059h
	.word  055h
	.word  051h
	.word  04dh
	.word  049h
	.word  045h
	.word  041h
	.word  03dh
	.word  039h
	.word  035h
	.word  031h
	.word  02dh
	.word  029h
	.word  025h
	.word  021h
	.word  01eh
	.word  01ch
	.word  01ah
	.word  018h
	.word  016h
	.word  014h
	.word  012h
	.word  010h
	.word  0eh
	.word  0ch
	.word  0ah
	.word  08h
	.word  06h
	.word  04h
	.word  02h
	.word  00h

	.end
