************************************************************************
*
*    ECHO30 - TMS320C30 256-TAP ECHO-CANCELLER PROGRAM WITH
*	      ONE WAIT STATE EXTERNAL MEMORY (DMA & CACHE)
*	      (C) COPYRIGHT TEXAS INSTRUMENTS INC., 1989
*
*
*    NOTE:    1. THE PROGRAM IS LOADED BY CONCURRENT DMA FROM EXTERNAL
*		 MEMORY TO INTERNAL RAM WHILE INITIALING THE Y[], A[],
*		 AND M[] ARRAY.
*	      2. LINK COMMAND FILE IS "ECHO30.CMD"
*
*	      DANIEL CHEIN-CHUNG CHEN	    OCTOBER, 1989
*
************************************************************************
*
*    ALGORITHM CONSTANTS
*
STEP_SIZE	  .set	   0.00029296875 ; Step size mu
STACK_SIZE	  .set	   40h		 ; Size of system stack
cutoff		  .set	   33.0 	 ; Cutoff constant
y_length	  .set	   256		 ; Filter y[] buffer size
M_block 	  .set	   17		 ; Buffer size for M[]
THRESHOLD	  .set	   0.0625	 ; Supress threshold = 1/16 = -24 dB
hangover	  .set	   600		 ; Hang over time = 75 mS (600*8KHz)
PRIMARY_CTL	  .set	   0B38H	 ; Primary bus control word for 1 wait
SP_RCV_CTL_WORD   .set	   0111H	 ; Serial port receive control word
SP_XMT_CTL_WORD   .set	   0171H	 ; Serial port transmit control word
SP_GLOBAL_CTL	  .set	   7400H	 ; Serial port global control wrod
					 ; bit 13 to bit 28 of 0E800000H

**************************************************************
*							     *
*			MEMORY ALLOCATION		     *
*							     *
**************************************************************
*
*    SERIAL PORT 0 REGISTER
*
SP0_GLOBAL_REG	  .usect   "s_port0",2
SP0_XMT_CTL_REG   .usect   "s_port0",1
SP0_RCV_CTL_REG   .usect   "s_port0",1
SP0_TIMER_CTL_REG .usect   "s_port0",1
SP0_TIMER_COUNTER .usect   "s_port0",1
SP0_TIMER_PERIOD  .usect   "s_port0",2
SP0_XMT_DATA	  .usect   "s_port0",4
SP0_RCV_DATA	  .usect   "s_port0",4
*
*    SERIAL PORT 1 REGISTER
*
SP1_GLOBAL_REG	  .usect   "s_port1",2
SP1_XMT_CTL_REG   .usect   "s_port1",1
SP1_RCV_CTL_REG   .usect   "s_port1",1
SP1_TIMER_CTL_REG .usect   "s_port1",1
SP1_TIMER_COUNTER .usect   "s_port1",1
SP1_TIMER_PERIOD  .usect   "s_port1",2
SP1_XMT_DATA	  .usect   "s_port1",4
SP1_RCV_DATA	  .usect   "s_port1",4
*
*    EXTERNAL BUS CONTROL REGISTER
*
EXPANSION_REG	  .usect   "ex_bus",4
PRIMARY_REG	  .usect   "ex_bus",4
*
*    DMA CONTROL REGISTER
*
DMA_GLOBAL_REG	  .usect   "DMA_bus",4
DMA_SOURCE_REG	  .usect   "DMA_bus",2
DMA_DESTIN_REG	  .usect   "DMA_bus",2
DMA_COUNT_REG	  .usect   "DMA_bus",2
*
*    SOFTWARE STACK AND BUFFERS
*
stack		  .usect   ".stack",STACK_SIZE
y_array 	  .usect   "buffer",y_length
M_array 	  .usect   "Marray",M_block
coefficients	  .usect   "coeffs",y_length
*
*    VARIABLE MEMORY
*
MU_BASE 	  .usect   "variable",1
VAR1_ADDR	  .usect   "variable",1
VAR2_ADDR	  .usect   "variable",1
COEFS_ADDR	  .usect   "variable",1
r2_1		  .usect   "variable",1
r_1		  .usect   "variable",1
ABSY0		  .usect   "variable",1
HCNTR		  .usect   "variable",1
Ei		  .usect   "variable",1
ABSEi		  .usect   "variable",1
a_1		  .usect   "variable",1
a		  .usect   "variable",1
b_1		  .usect   "variable",1
b		  .usect   "variable",1
b2		  .usect   "variable",1
SDCi		  .usect   "variable",1
Si		  .usect   "variable",1
ABSSi		  .usect   "variable",1
IABSY		  .usect   "variable",1
ABSOUT		  .usect   "variable",1
ABSY		  .usect   "variable",1
ABSY0F		  .usect   "variable",1
ABSS0F		  .usect   "variable",1
GAIN		  .usect   "variable",1

	.page
**************************************************************
*							     *
*		      INTERRUPT VECTORS 		     *
*							     *
**************************************************************

		  .sect    "vectors"
RESET		  .word    INIT       ; On hardware RESET go to INIT

		  .space   5
RINT0		  .word    809F00H+RINT0_ROUT-CONST
				      ; On hardware RINT0 go to RINT0_ROUT
XINT1		  .word    809F00H+XINT1_ROUT-CONST
				      ; On hardware XINT1 go to XINT1_ROUT
RINT1		  .word    809F00H+RINT1_ROUT-CONST
				      ; On hardware RINT1 go to RINT1_ROUT
		  .space   2
DINT		  .word    809F00H+START-CONST
				      ; Dint vector address

**************************************************************
*							     *
*	       PROCESSOR INITIALIZATION ROUTINE 	     *
*							     *
**************************************************************

		  .text
CONST_addr	  .word    CONST       ; Address of CONST
*
*    SET PRIMARY BUS WAIT STATE
*
INIT:	 LDI	80h,DP		      ; Data page = on-chip peripherals, on-chip
				      ; 	    RAM, and expansion bus
	 LDI	PRIMARY_CTL,R0	      ; Set primary bus wait state
	 STI	R0,@PRIMARY_REG
*
*    SET UP DMA TO LOAD THE PROGRAM INTO RAM
*
	 LDI	0C0H,AR2	      ; Load the source address
	 LDI	*AR2,AR1
	 STI	AR1,@DMA_SOURCE_REG
	 LDI	404FH,AR0	      ; Load the destination address
	 LSH	9,AR0
	 ADDI	100H,AR0
	 STI	AR0,@DMA_DESTIN_REG
	 LDI	LOOP-CONST+4,AR2      ; Load the DMA counter register
	 STI	AR2,@DMA_COUNT_REG
	 LDI	0C53H,AR2	      ; Set the DMA global control register
	 STI	AR2,@DMA_GLOBAL_REG
*
*    ENABLE THE PRAGROM CACHE AND GLOBAL INTERRUPT
*
	 OR	2800,ST 	      ; Turn on Cache (11) and GIE (13) bits
*
*    INITIALIZE THE Y[], A[] AND M[] buffers
*
	 LDI	AR0,AR2 	      ; Get A[] buffer address
	 SUBI	300H,AR2
	 LDI	AR2,AR3 	      ; Get Y[] buffer address
	 SUBI	400H,AR3
	 LDF	0.0,R1		      ; R1 = 0.0
	 RPTS	y_length-1	      ; Repeat y_length times
	 STF	R1,*AR2++(1)	      ; Zero Y[] buffer
      || STF	R1,*AR3++(1)	      ; Zero A[] buffer

	 LDI	AR0,AR2 	      ; Get M[] buffer address
	 SUBI	400H,AR2
	 RPTS	M_block-1	      ; Repeat M_block times
	 STF	R1,*AR2++(1)	      ; Zero M[] buffer
*
*    ENABLE DINT INTERRUPT AND WAIT FOR THE DINT
*
	 LDI	400H,IE
	 IDLE
*
*    CONSTANTS FOR VARIABLES
*
CONST:	 .word	  MU_TABLE
	 .word	  r2_1
	 .word	  SDCi
	 .word	  coefficients
	 .float   0.9375
	 .float   0.875
	 .float   0.0
	 .word	  0
	 .float   0.0
	 .float   0.0
	 .float   0.9921875
	 .float   0.0078125
	 .float   0.96875
	 .float   0.03125
	 .float   0.0625
	 .float   0.0
	 .float   0.0
	 .float   0.0
	 .float   0.0009765625
	 .float   0.0
	 .float   1024.0
	 .float   1024.0
	 .float   0.0
	 .float   STEP_SIZE
*
*    SET UP SERIAL PORTS
*
START:	 LDI	0,R0		      ; Reset serial ports and timers
	 STI	R0,@SP0_GLOBAL_REG
	 STI	R0,@SP1_GLOBAL_REG
	 STI	R0,@SP0_TIMER_CTL_REG
	 STI	R0,@SP1_TIMER_CTL_REG

	 LDI	SP_RCV_CTL_WORD,R0    ; Set serial port 0 receive pin function
	 STI	R0,@SP0_RCV_CTL_REG
	 STI	R0,@SP1_RCV_CTL_REG   ; Set serial port 1 receive pin function
	 LDI	SP_XMT_CTL_WORD,R0    ; Set serial port 1 transmit pin function
	 STI	R0,@SP1_XMT_CTL_REG

	 LDI	SP_GLOBAL_CTL,R0      ; external clock and framing sources, etc.
	 LSH	13,R0		      ; Global control word = 0E800000h
	 STI	R0,@SP0_GLOBAL_REG    ; Set serial port 0 global control
	 STI	R0,@SP1_GLOBAL_REG    ; Set serial port 1 global control

	 LDI	@SP0_RCV_DATA,R7      ; Dummy read for serial receive registers
	 LDI	@SP1_RCV_DATA,R7
	 LDI	0FFFFH,R7	      ; Initialize receive registers
	 LDI	R7,R6
	 STI	R7,@SP1_XMT_DATA      ; Initialize serial port transmit register

*
*    SET UP BK, H COUNTER AND STACK POINTER
*
	 LDI	y_length,BK	      ; Set circular buffer size
	 LDI	1,IR0		      ; Initialize H counter
	 LDI	AR0,SP		      ; Set up stack pointer SP
	 SUBI	0340H,SP
*
*    INITIALIZE M0 AND VARIABLES POONTER 1, 2
*
	 LDI	AR0,AR7 	      ; Initialize Y[] pointer
	 SUBI	0700H,AR7
	 LDI	AR0,AR6 	      ; Initialize A[] pointer
	 SUBI	0300H,AR6
	 SUBI	0400H,AR0	      ; Initialize M[] pointer
	 LDI	@VAR1_ADDR,AR1	      ; Initialize variables pointer 1
	 LDI	@VAR2_ADDR,AR2	      ; Initialize variables pointer 2
	 LDF	@ABSY0F,R1	      ; R1 = 1024.0
	 STF	R1,*AR0 	      ; Initialize M0
	 LDI	0E0H,IE 	      ; Enabled serial port interrupts

	 B	LOOP+1


	 .page
**************************************************************
*							     *
*	     RECEIVE INTERRUPT SERVICE ROUTINE		     *
*							     *
**************************************************************

RINT1_ROUT:			      ; (far-end signal)
	 PUSH	ST		      ; Save status register
	 PUSH	AR4		      ; Save AR4 register
	 LDI	@SP1_RCV_DATA,AR4     ; Input mu-law value
	 AND	0FFH,AR4	      ; Mask-off MSB byte
	 ADDI	@MU_BASE,AR4	      ; Add mu-law table base address
	 LDF	*AR4,R7 	      ; Mu-law to linear
	 POP	AR4		      ; Restore AR4 register
	 POP	ST		      ; Restore status register
	 RETI


RINT0_ROUT:			      ; (near-end signal)
	 PUSH	ST		      ; Save status register
	 PUSH	AR4		      ; Save AR4 register
	 LDI	@SP0_RCV_DATA,AR4     ; Input mu-law value
	 AND	0FFH,AR4	      ; Mask-off MSB byte
	 ADDI	@MU_BASE,AR4	      ; Add mu-law table base address
	 LDF	*AR4,R6 	      ; Mu-law to linear
	 POP	AR4		      ; Restore AR4 register
	 POP	ST		      ; Restore status register
	 RETI


**************************************************************
*							     *
*	     RECEIVE INTERRUPT SERVICE ROUTINE		     *
*							     *
**************************************************************

XINT1_ROUT:
	 OR	2000H,ST	       ; Interrupts enabled
	 LDI	@COEFS_ADDR,AR6        ; Initialize A[] pointer

**************************************************************
*		 ECHO ESTIMATION ROUTINE		     *
**************************************************************

ECHO_ESTIMATE:
	 LDF	0.0,R2		       ; Clear R2
	 MPYF3	R7,*AR6++(1),R0
      || STF	R7,*AR7++(1)%	       ; y(0) --> Y[]

	 RPTS	y_length-3	       ; Repeat 254 times
	 MPYF3	*AR6++(1),*AR7++(1)%,R0
      || ADDF3	R0,R2,R2	       ; SUM Ai(k) * y(i-k)  k = 0, 254

	 MPYF3	*AR6,*AR7,R0	       ; R0 = Ai(255) * y(i-255)
      || ADDF3	R0,R2,R2
	 ADDF3	R0,R2,R0	       ; Include last Ai(k)*y(i-k)

**************************************************************
*	    COMPUTE HIGH PASS FILTERED NEAR SAMPLE	     *
**************************************************************

	 ABSF	R7,R5		       ; |y(0)|
	 SUBF3	*AR2,R6,R2	       ; R2 = s0dc - s1dc
	 MPYF3	R2,*AR1++(1),R2        ; (1-r2)*[s0dc-s1dc]
      || STF	R6,*AR2++(1)	       ; Store s1dc
	 MPYF3	*AR1++(1),*AR2,R1      ; (1-r) * s(i)
	 ADDF3	R1,R2,R3	       ; Calculate s(i+1)
	 ABSF	R3,R2		       ; |s(i+1)|
	 STF	R3,*AR2++(2)	       ; Store s(i+1)
	 STF	R2,*-AR2(1)	       ; Store |s(i+1)|
      || STF	R5,*AR1++(1)	       ; Store |y(0)|

**************************************************************
*		  OUTPUT ESTIMATION ROUTINE		     *
**************************************************************
*
*    CHECK NEAR END SPEECH
*
	 LDI	*AR1++(2),R1	       ; Check N.E. speech flag (hangover)
	 BGTD	NO_SPRS
*
*    COMPUTE THE OUTPUT
*
	 MPYF3	*AR2,*++AR2(1),R1      ; ABSOUT(i)/ABSY(i)
      || SUBF3	R0,R3,R3	       ; R3 = OUTPUT
	 ABSF	R3,R2		       ; Compute |E(i)|
	 STF	R3,*-AR1(1)	       ; Store E(i)
      || STF	R2,*AR1++(1)	       ; Store |E(i)|

**************************************************************
*	    RESIDUAL OUTPUT SUPPRESSION ROUTINE 	     *
**************************************************************

	 CMPF	THRESHOLD,R1	       ; Check if signal > threshold
	 LDFLE	0.0,R3		       ; Zero output if signal < threshold

**************************************************************
*	    LINEAR TO MU-LAW COMPRESSION ROUTINE	     *
**************************************************************

NO_SPRS  FIX	R3,R4		       ; Save sign and convert to integer
	 ABSI	R4,R1
	 CMPI	1FDEH,R1	       ; If R1 > 1FDEh
	 LDIGT	1FDEH,R1	       ; Saturate the result
	 ADDI	33,R1		       ; Add bias

	 FLOAT	R1		       ; Normalize: (seg+5)0WXYZx...x
	 MPYF	0.03125,R1	       ; Adjust segment number by 2^-5
	 LSH	1,R1		       ; (seg)WXYZx...x
	 PUSHF	R1
	 POP	R1		       ; Treat number as integer
	 LSH	-20,R1		       ; Right-justify

	 LDI	0,R3
	 LDI	R4,R4		       ; If the number is negative
	 LDILT	80H,R3		       ; Set sign bit
	 ADDI	R3,R1		       ; R0 = compressed number
	 NOT	R1		       ; Reverse all bits for transmission
OUTPUT:  STI	R1,@SP1_XMT_DATA       ; Store in serial port transmit register

**************************************************************
*		   POWER ESTIMATION ROUTINE		     *
**************************************************************
*
*    UPDATE LONG TAU OUTPUT AND REFERENCE POWER ESTIMATE
*
	 MPYF3	*AR1++(1),*AR2++(1),R4 ; R4 = (1-a) * ABSOUT(i)
	 MPYF3	R2,*AR1,R2	       ; a * |E(i)|
	 MPYF3	*-AR1(1),*AR2,R1       ; R1 = (1-a) * ABSY(i)
      || ADDF3	R4,R2,R2	       ; ABSOUT(i+1)
	 ADDF	cutoff,R5	       ; |Y(0)| + cutoff
	 MPYF	*AR1++(1),R5	       ; a * (|Y(0)|+cutoff)
	 ADDF3	R5,R1,R3	       ; ABSY(i+1)
	 STF	R2,*-AR2(1)	       ; Store ABSOUT(i+1)
      || STF	R3,*AR2++(1)	       ; Store ABSY(i+1)
*
*    COMPUTE 1/ABSY (DEVIDE 1 BY ABSY
*
	 LDI	*-AR2(1),R2	       ; var(n) = a * 2**e
	 ASH	-24,R2
	 NEGI	R2
	 SUBI	1,R2		       ; Now we have -e-1, the exponent of x[0].
	 ASH	24,R2
	 PUSH	R2
	 POPF	R2		       ; Now R2 = x[0] = 1.0 * 2**(-e-1).
*
	 MPYF	R2,R3,R4	       ; R4 = v * x[0]
	 SUBRF	2.0,R4		       ; R4 = 2.0 - v * x[0]
	 MPYF	R4,R2		       ; R2 = x[1] = x[0] * (2.0 - v * x[0])

	 MPYF	R2,R3,R4	       ; R4 = v * x[1]
	 SUBRF	2.0,R4		       ; R4 = 2.0 - v * x[1]
	 MPYF	R4,R2		       ; R2 = x[2] = x[1] * (2.0 - v * x[1])

	 MPYF	R2,R3,R4	       ; R4 = v * x[2]
	 SUBRF	2.0,R4		       ; R4 = 2.0 - v * x[2]
	 MPYF	R4,R2		       ; R2 = x[3] = x[2] * (2.0 - v * x[2])

	 MPYF	R2,R3,R4	       ; R4 = v * x[3]
	 SUBRF	2.0,R4		       ; R4 = 2.0 - v * x[3]
	 MPYF	R4,R2		       ; R2 = x[4] = x[3] * (2.0 - v * x[3])

	 RND	R2		       ; This minimizes error in the LSBs.
*
	 MPYF	R2,R3,R4	       ; R4 = v * x[4] = 1.0..01.. => 1
	 SUBRF	1.0,R4		       ; R4 = 1.0 - v * x[4] = 0.0..01... => 0
	 MPYF	R2,R4		       ; R4 = x[4] * (1.0 - v * x[4])
	 ADDF	R4,R2		       ; R2 = x[5] = (x[4]*(1.0-(v*x[4])))+x[4]

	 RND	R2,R4		       ; Round since this is follow by a MPYF.
	 STF	R4,*-AR2(3)	       ; Store 1/ABSY

**************************************************************
*		OUTPUT NORMALIZATION ROUTINE		     *
**************************************************************

	 MPYF3	R4,R4,R5	       ; R5 = 1/(ABSY)**2
	 MPYF	*-AR1(4),R5	       ; R5 = u(i)/(ABSY)**2
	 MPYF	@GAIN,R5	       ; R5 = GAIN*u(i)/(ABSY)**2

**************************************************************
*	      NEAR END SPEECH DETECTION ROUTINE 	     *
**************************************************************
*
*    UPDATE SHORT TAU NEAR END AND REFERENCE POWER ESTIMATE
*
	 MPYF3	*AR1++(1),*AR2++(1),R4 ; R4 = (1-b) * ABSY0F(i)
	 LDF	*-AR1(7),R2	       ; R2 = |Y(0)|
	 MPYF	*AR1,R2 	       ; R2 = b * |Y(0)|
	 MPYF3	*-AR1(1),*AR2,R1       ; R1 = (1-b) * ABSS0F(i)
      || ADDF3	R2,R4,R2	       ; R2 = ABSY0F(i+1)
	 LDF	*-AR2(5),R3	       ; R3 = |S(i)|
	 MPYF	*++AR1(1),R3	       ; R3 = 2b * |S(i)|
	 ADDF	R1,R3		       ; R3 = ABSS0F(i+1)
	 STF	R2,*-AR2(1)	       ; Store ABSY0F(i+1)
      || STF	R3,*AR2++(1)	       ; Store ABSS0F(i+1)
*
*    UPDATE MODULO 16 COUNTER (H)
*
	 LDI	IR0,R1		       ; R1 = COUNTER H
	 BGTD	NEXT1		       ; Delay branch if H > 0
	 LDI	M_block,BK	       ; Set circular buffer size
	 ADDI	1,IR0		       ; H = H + 1
	 AND	0FH,IR0 	       ; If H = 16 then H = 0
*
*    UPDATE LOACAL MAXIMUM M0, M1, ....., M16
*
	 BD	NEXT2		       ; Delay branch
	 NOP	*AR0--(1)%	       ; Update M[] pointer
	 STF	R4,*AR0 	       ; Update M[0]
	 LDI	*-AR1(7),R1	       ; Load hang over counter
*
*    UPDATE MOST RECENT LOCAL MAXIMA M0
*
NEXT1:	 CMPF	*AR0,R2 	       ; If ABSY0F > M0
	 LDFLE	*AR0,R2 	       ; R4 = max(ABSY0F,M0)
	 STF	R2,*AR0 	       ; M0 = R4
*
*    COMPARE REFERENCE POWER TO NEAR END POWER
*
	 LDI	hangover,R1	       ; R3 = hangover reset
	 LDI	M_block-1,RC	       ; Set the repeat counter
	 RPTB	809F00H+POWER-CONST
	 CMPF	*AR0++(1)%,R3	       ; Compare ABSS0F with Mi
POWER:	 LDILE	*-AR1(7),R1	       ; R1 = hangover counter if ABSS0F <= Mi
*
*    CHECK AND UPDATE HANGOVER COUNTER
*
NEXT2:	 CMPI	0,R1		       ; Check if hangover counter = 0
	 BEQD	NEXT3		       ; Delay branch on R3 = 0
	 LDI	y_length,BK	       ; Set circular buffer size
	 LDF	*-AR2(3),R2	       ; R2 = ABSY
	 NOP	*AR2--(8)	       ; Reset the variable pointer

	 NOP	*AR1--(10)	       ; Reset the variable pointer
	 SUBI	1,R1		       ; HCNTR = HCNTR - 1
	 STI	R1,*+AR1(3)	       ; Rstore HCNTR
	 B	LOOP		       ; Go to cycle end

NEXT3:	 CMPF	cutoff,R2	       ; Compare ABSY with cutoff
	 BLED	LOOP		       ; Go to cycle end if ABSY <= cutoff
	 NOP	*AR1--(10)	       ; Reset the variable pointer
	 LDI	y_length-2,RC	       ; Set loop counter
	 NOP

	 MPYF3	R5,*AR7--(1)%,R0       ; R0 = u(i) * y(n)

	 RPTB	809F00H+COEF-CONST
	 MPYF3	R5,*AR7--(1)%,R0       ; R0 = u(i) * y(n)
      || ADDF3	R0,*AR6,R2
COEF:	 STF	R2,*AR6--(1)	       ; Restore Ai+1[]

	 ADDF3	R0,*AR6,R2
	 STF	R2,*AR6 	       ; Restore Ai+1[]

LOOP:	 SUBI	1,SP		       ; Update SP since there is no
				       ; return from this interrupt
	 IDLE			       ; Wait for interrupt
	 IDLE			       ; Wait for interrupt
	 IDLE


MU_TABLE:
         .float     -8031.000000
         .float     -7775.000000
         .float     -7519.000000
         .float     -7263.000000
         .float     -7007.000000
         .float     -6751.000000
         .float     -6495.000000
         .float     -6239.000000
         .float     -5983.000000
         .float     -5727.000000
         .float     -5471.000000
         .float     -5215.000000
         .float     -4959.000000
         .float     -4703.000000
         .float     -4447.000000
         .float     -4191.000000
         .float     -3999.000000
         .float     -3871.000000
         .float     -3743.000000
         .float     -3615.000000
         .float     -3487.000000
         .float     -3359.000000
         .float     -3231.000000
         .float     -3103.000000
         .float     -2975.000000
         .float     -2847.000000
         .float     -2719.000000
         .float     -2591.000000
         .float     -2463.000000
         .float     -2335.000000
         .float     -2207.000000
         .float     -2079.000000
         .float     -1983.000000
         .float     -1919.000000
         .float     -1855.000000
         .float     -1791.000000
         .float     -1727.000000
         .float     -1663.000000
         .float     -1599.000000
         .float     -1535.000000
         .float     -1471.000000
         .float     -1407.000000
         .float     -1343.000000
         .float     -1279.000000
         .float     -1215.000000
         .float     -1151.000000
         .float     -1087.000000
         .float     -1023.000000
         .float     -975.000000
         .float     -943.000000
         .float     -911.000000
         .float     -879.000000
         .float     -847.000000
         .float     -815.000000
         .float     -783.000000
         .float     -751.000000
         .float     -719.000000
         .float     -687.000000
         .float     -655.000000
         .float     -623.000000
         .float     -591.000000
         .float     -559.000000
         .float     -527.000000
         .float     -495.000000
         .float     -471.000000
         .float     -455.000000
         .float     -439.000000
         .float     -423.000000
         .float     -407.000000
         .float     -391.000000
         .float     -375.000000
         .float     -359.000000
         .float     -343.000000
         .float     -327.000000
         .float     -311.000000
         .float     -295.000000
         .float     -279.000000
         .float     -263.000000
         .float     -247.000000
         .float     -231.000000
         .float     -219.000000
         .float     -211.000000
         .float     -203.000000
         .float     -195.000000
         .float     -187.000000
         .float     -179.000000
         .float     -171.000000
         .float     -163.000000
         .float     -155.000000
         .float     -147.000000
         .float     -139.000000
         .float     -131.000000
         .float     -123.000000
         .float     -115.000000
         .float     -107.000000
         .float     -99.000000
         .float     -93.000000
         .float     -89.000000
         .float     -85.000000
         .float     -81.000000
         .float     -77.000000
         .float     -73.000000
         .float     -69.000000
         .float     -65.000000
         .float     -61.000000
         .float     -57.000000
         .float     -53.000000
         .float     -49.000000
         .float     -45.000000
         .float     -41.000000
         .float     -37.000000
         .float     -33.000000
         .float     -30.000000
         .float     -28.000000
         .float     -26.000000
         .float     -24.000000
         .float     -22.000000
         .float     -20.000000
         .float     -18.000000
         .float     -16.000000
         .float     -14.000000
         .float     -12.000000
         .float     -10.000000
         .float     -8.000000
         .float     -6.000000
         .float     -4.000000
         .float     -2.000000
         .float     0.000000
         .float     8031.000000
         .float     7775.000000
         .float     7519.000000
         .float     7263.000000
         .float     7007.000000
         .float     6751.000000
         .float     6495.000000
         .float     6239.000000
         .float     5983.000000
         .float     5727.000000
         .float     5471.000000
         .float     5215.000000
         .float     4959.000000
         .float     4703.000000
         .float     4447.000000
         .float     4191.000000
         .float     3999.000000
         .float     3871.000000
         .float     3743.000000
         .float     3615.000000
         .float     3487.000000
         .float     3359.000000
         .float     3231.000000
         .float     3103.000000
         .float     2975.000000
         .float     2847.000000
         .float     2719.000000
         .float     2591.000000
         .float     2463.000000
         .float     2335.000000
         .float     2207.000000
         .float     2079.000000
         .float     1983.000000
         .float     1919.000000
         .float     1855.000000
         .float     1791.000000
         .float     1727.000000
         .float     1663.000000
         .float     1599.000000
         .float     1535.000000
         .float     1471.000000
         .float     1407.000000
         .float     1343.000000
         .float     1279.000000
         .float     1215.000000
         .float     1151.000000
         .float     1087.000000
         .float     1023.000000
         .float     975.000000
         .float     943.000000
         .float     911.000000
         .float     879.000000
         .float     847.000000
         .float     815.000000
         .float     783.000000
         .float     751.000000
         .float     719.000000
         .float     687.000000
         .float     655.000000
         .float     623.000000
         .float     591.000000
         .float     559.000000
         .float     527.000000
         .float     495.000000
         .float     471.000000
         .float     455.000000
         .float     439.000000
         .float     423.000000
         .float     407.000000
         .float     391.000000
         .float     375.000000
         .float     359.000000
         .float     343.000000
         .float     327.000000
         .float     311.000000
         .float     295.000000
         .float     279.000000
         .float     263.000000
         .float     247.000000
         .float     231.000000
         .float     219.000000
         .float     211.000000
         .float     203.000000
         .float     195.000000
         .float     187.000000
         .float     179.000000
         .float     171.000000
         .float     163.000000
         .float     155.000000
         .float     147.000000
         .float     139.000000
         .float     131.000000
         .float     123.000000
         .float     115.000000
         .float     107.000000
         .float     99.000000
         .float     93.000000
         .float     89.000000
         .float     85.000000
         .float     81.000000
         .float     77.000000
         .float     73.000000
         .float     69.000000
         .float     65.000000
         .float     61.000000
         .float     57.000000
         .float     53.000000
         .float     49.000000
         .float     45.000000
         .float     41.000000
         .float     37.000000
         .float     33.000000
         .float     30.000000
         .float     28.000000
         .float     26.000000
         .float     24.000000
         .float     22.000000
         .float     20.000000
         .float     18.000000
         .float     16.000000
         .float     14.000000
         .float     12.000000
         .float     10.000000
         .float     8.000000
         .float     6.000000
         .float     4.000000
         .float     2.000000
         .float     0.000000

	 .end






